@@ -10,6 +10,14 @@ pub enum ConnectionType {
     Tcp,
     #[cfg(feature = "tls")]
     Tls,
+    #[cfg(feature = "http")]
+    HttpGet,
+    #[cfg(feature = "http")]
+    HttpPost,
+    #[cfg(feature = "http")]
+    HttpsGet,
+    #[cfg(feature = "http")]
+    HttpsPost,
 }
 
 #[derive(Debug)]
@@ -17,9 +25,11 @@ pub struct Args {
     pub nameserver: String,
     pub url: String,
     pub qtype: DnsType,
-    pub short: bool,
+    pub verbose: bool,
     #[cfg(feature = "json")]
     pub json: bool,
+    pub print_meta: bool,
+    pub pad_answers: bool,
     pub use_dnssec: bool,
     pub connection_type: ConnectionType,
     pub port: u16,
@@ -29,7 +39,7 @@ enum ConsumeNext {
     Port,
 }
 
-const DEFAULT_NAMESERVER: &str = "1.1.1.1";
+const DEFAULT_NAMESERVER: &str = "ordns.he.net";
 const DEFAULT_URL: &str = "example.com";
 const DEFAULT_QTYPE: DnsType = DnsType::AAAA;
 
@@ -40,9 +50,11 @@ pub fn parse_args() -> Args {
     let mut nameserver = DEFAULT_NAMESERVER.into();
     let mut url = DEFAULT_URL.into();
     let mut qtype = DEFAULT_QTYPE;
-    let mut short = false;
+    let mut verbose = false;
     #[cfg(feature = "json")]
     let mut json = false;
+    let mut print_meta = true;
+    let mut pad_answers = true;
     let mut use_dnssec = false;
     let mut connection_type = ConnectionType::Udp;
     let mut port = None;
@@ -65,13 +77,19 @@ pub fn parse_args() -> Args {
         } else if let Some(flag) = arg.strip_prefix('+') {
             // flags
             match flag {
-                "short" => {
-                    short = true;
+                "verbose" => {
+                    verbose = true;
                 }
                 #[cfg(feature = "json")]
                 "json" => {
                     json = true;
                 }
+                "no-meta" => {
+                    print_meta = false;
+                }
+                "no-padding" => {
+                    pad_answers = false;
+                }
                 "do" => {
                     use_dnssec = true;
                 }
@@ -79,11 +97,27 @@ pub fn parse_args() -> Args {
                     connection_type = ConnectionType::Tcp;
                 }
                 #[cfg(feature = "tls")]
-                "tls" => {
+                "dot" | "tls" => {
                     connection_type = ConnectionType::Tls;
                 }
+                #[cfg(feature = "http")]
+                "http-get" => {
+                    connection_type = ConnectionType::HttpGet;
+                }
+                #[cfg(feature = "http")]
+                "http" | "http-post" => {
+                    connection_type = ConnectionType::HttpPost;
+                }
+                #[cfg(feature = "http")]
+                "https-get" => {
+                    connection_type = ConnectionType::HttpsGet;
+                }
+                #[cfg(feature = "http")]
+                "doh" | "https" | "https-post" => {
+                    connection_type = ConnectionType::HttpsPost;
+                }
                 x => {
-                    err(format!("Invalid flag: {}.", x));
+                    err(format!("Invalid flag: +{}.", x));
                 }
             }
         } else if let Some(option) = arg.strip_prefix('-') {
@@ -105,7 +139,6 @@ pub fn parse_args() -> Args {
                 }
                 x => {
                     err(format!("Invalid option: -{}.", x));
-                    process::exit(1);
                 }
             }
         } else {
@@ -121,6 +154,10 @@ pub fn parse_args() -> Args {
         }
     }
 
+    if verbose && !pad_answers {
+        err("Cannot use both +verbose and +no-padding.");
+    }
+
     if reverse {
         match IpAddr::from_str(url.as_str()) {
             Err(_) => {
@@ -160,13 +197,24 @@ pub fn parse_args() -> Args {
         }
     }
 
+    #[cfg(feature = "http")]
+    if port.is_none() {
+        if [ConnectionType::HttpGet, ConnectionType::HttpPost].contains(&connection_type) {
+            port = Some(80);
+        } else if [ConnectionType::HttpsGet, ConnectionType::HttpsPost].contains(&connection_type) {
+            port = Some(443);
+        }
+    }
+
     Args {
         nameserver,
         url,
         qtype,
-        short,
+        verbose,
         #[cfg(feature = "json")]
         json,
+        print_meta,
+        pad_answers,
         use_dnssec,
         connection_type,
         port: port.unwrap_or(53),
@@ -185,13 +233,28 @@ fn print_help() {
     println!("       \t    -x                     (shortcut for reverse lookup)");
     println!();
     println!("       \tflags is one or more of the following:");
-    println!("       \t    +short  (only print answer sections of response)");
+    println!("       \t    +verbose     (print all sections, i.e. header, OPT, and question)");
     #[cfg(feature = "json")]
-    println!("       \t    +json   (format output as JSON)");
-    println!("       \t    +do     (fetch DNSSEC records)");
-    println!("       \t    +tcp    (use TCP instead of UDP)");
+    println!("       \t    +json        (format output as JSON; may be used with +verbose)");
+    println!("       \t    +no-meta     (don't print query metadata, e.g. server and time)");
+    println!("       \t    +no-padding  (don't pad output; cannot be used with +verbose)");
+    println!("       \t    +do          (fetch DNSSEC records)");
+    println!("       \t    +tcp         (use TCP instead of UDP)");
     #[cfg(feature = "tls")]
-    println!("       \t    +tls    (use DNS over TLS)");
+    {
+        println!("       \t    +dot         (use DNS over TLS)");
+        println!("       \t    +tls         (use DNS over TLS)");
+    }
+    #[cfg(feature = "http")]
+    {
+        println!("       \t    +doh         (use DNS over HTTPS, with POST)");
+        println!("       \t    +https       (use DNS over HTTPS, with POST)");
+        println!("       \t    +https-post  (use DNS over HTTPS, with POST)");
+        println!("       \t    +https-get   (use DNS over HTTPS, with GET)");
+        println!("       \t    +http        (use DNS over HTTP, with POST)");
+        println!("       \t    +http-post   (use DNS over HTTP, with POST)");
+        println!("       \t    +http-get    (use DNS over HTTP, with GET)");
+    }
     println!();
     println!("Note: order of the arguments does not matter.");
     println!();
