@@ -6,6 +6,7 @@ use rand::Rng;
 use serde::Serialize;
 use std::fmt::{self, Display};
 use std::io::{Cursor, Read, Seek, SeekFrom};
+use std::net::{Ipv4Addr, Ipv6Addr};
 use std::{cmp::max, collections::HashMap};
 use strum_macros::EnumString;
 
@@ -150,6 +151,7 @@ pub struct DnsQuestion {
 /// message.
 /// See [RFC 1035](https://tools.ietf.org/pdf/rfc1035.pdf) for further information.
 #[cfg_attr(feature = "json", derive(Serialize))]
+#[cfg_attr(feature = "json", serde(untagged))]
 #[derive(Clone)]
 pub enum DnsRecord {
     OPT {
@@ -690,7 +692,7 @@ impl DnsQuestion {
             owner.push(' ');
         }
 
-        res.push_str(format!("{}         {}  {}", owner, self.qclass, self.qtype).as_str());
+        res.push_str(format!("{}         {}", owner, self.qtype).as_str());
 
         res
     }
@@ -833,21 +835,26 @@ impl DnsRecord {
                     format!("\"{}\"", DnsMessage::parse_string(msg)?)
                 }
                 "ip4" => {
-                    format!(
-                        "{}.{}.{}.{}",
+                    let addr = Ipv4Addr::new(
                         msg.read_u8()?,
                         msg.read_u8()?,
                         msg.read_u8()?,
-                        msg.read_u8()?
-                    )
+                        msg.read_u8()?,
+                    );
+                    addr.to_string()
                 }
                 "ip6" => {
-                    let mut addr = String::new();
-                    for _ in 0..8 {
-                        addr.push_str(format!("{:x}:", msg.read_u16::<NetworkEndian>()?).as_str());
-                    }
-                    addr.remove(addr.len() - 1); // remove last ':'
-                    addr
+                    let addr = Ipv6Addr::new(
+                        msg.read_u16::<NetworkEndian>()?,
+                        msg.read_u16::<NetworkEndian>()?,
+                        msg.read_u16::<NetworkEndian>()?,
+                        msg.read_u16::<NetworkEndian>()?,
+                        msg.read_u16::<NetworkEndian>()?,
+                        msg.read_u16::<NetworkEndian>()?,
+                        msg.read_u16::<NetworkEndian>()?,
+                        msg.read_u16::<NetworkEndian>()?,
+                    );
+                    addr.to_string()
                 }
                 "text" => {
                     let mut s = String::new();
@@ -999,14 +1006,13 @@ impl DnsRecord {
     }
 
     /// Returns a string representing the record in the canonical format, with the
-    /// owner padded to the given length.
+    /// owner and atype fields padded to the given lengths.
     /// Panics when called on an OPT record.
     pub fn as_padded_string(&self, owner_len: usize, atype_len: usize) -> String {
         match self {
             DnsRecord::NONOPT {
                 name,
                 atype,
-                class,
                 ttl,
                 parsed_rdata,
                 ..
@@ -1025,10 +1031,9 @@ impl DnsRecord {
 
                 res.push_str(
                     format!(
-                        "{}  {:>5}  {}  {}  {}",
+                        "{}  {:>5}  {}  {}",
                         owner,
                         ttl,
-                        class,
                         atype,
                         DnsRecord::format_parsed_rdata(parsed_rdata)
                     )
@@ -1141,18 +1146,16 @@ impl Display for DnsRecord {
             DnsRecord::NONOPT {
                 name,
                 atype,
-                class,
                 ttl,
                 parsed_rdata,
                 ..
             } => {
                 write!(
                     f,
-                    "DNS Record for '{}' (type: {}, class: {}, ttl: {}, rdata: {})",
+                    "{} {} {} {}",
                     name,
-                    atype,
-                    class,
                     ttl,
+                    atype,
                     DnsRecord::format_parsed_rdata(parsed_rdata)
                 )
             }
